//*****************************************************
// A/D convert -> FFT -> Results on LCD
// 7.37MHz Internal RC oscillator, 16x PLL enabled
// Fcy=7.37MHzx16/4=29.48MHz, Tcy=33.92ns
// Toshio Iwata at DIGITALFILTER.COM all rights reserved
// Last modified on 2008/08/05
//*****************************************************

#include <p30f2012.h>
#include <adc12.h>
#include <timer.h>
#include <dsp.h>
#include "xlcd.h"
#include <stdio.h>

#define powerOf2		6
#define fftPoints	(1<<powerOf2) // 64|CgFFTADCTvO16kHzȂ̂Ŏg\16k/64=250Hz

// ]q̒`
const fractcomplex twiddleFactors[] __attribute__ ((space(auto_psv), aligned (fftPoints * 2))) = {
	0x7FFF, 0x0000, 0x7F62, 0xF374, 0x7D8A, 0xE707, 0x7A7D, 0xDAD8,
	0x7642, 0xCF04, 0x70E3, 0xC3A9, 0x6A6E, 0xB8E3, 0x62F2, 0xAECC,
	0x5A82, 0xA57E, 0x5134, 0x9D0E, 0x471D, 0x9592, 0x3C57, 0x8F1D,
	0x30FC, 0x89BE, 0x2528, 0x8583, 0x18F9, 0x8276, 0x0C8C, 0x809E,
	0x0000, 0x8000, 0xF374, 0x809E, 0xE707, 0x8276, 0xDAD8, 0x8583,
	0xCF04, 0x89BE, 0xC3A9, 0x8F1D, 0xB8E3, 0x9592, 0xAECC, 0x9D0E,
	0xA57D, 0xA57D, 0x9D0E, 0xAECC, 0x9592, 0xB8E3, 0x8F1D, 0xC3A9,
	0x89BE, 0xCF04, 0x8583, 0xDAD8, 0x8276, 0xE707, 0x809E, 0xF374
} ;

// ̓obt@ij
fractional	inputData[ fftPoints ];

// XyNg(f) 
fractcomplex fftData[ fftPoints ] __attribute__ ((section (".ybss, bss, ymemory"), aligned (fftPoints * 2 * 2)));

// p[XyNgij
fractional	powerSpec[ fftPoints/2 ];		

// configuration
_FWDT(WDT_OFF);
_FGS(CODE_PROT_OFF);
_FOSC(CSW_FSCM_OFF & FRC_PLL16);
_FBORPOR(PBOR_OFF & PWRT_64 & MCLR_EN);

// A/D control register
unsigned int _ADCON1;
unsigned int _ADCON2;
unsigned int _ADCON3;
unsigned int _ADCHS;
unsigned int _ADPCFG;
unsigned int _ADCSSL;

// data
unsigned int resultData;
int indata;
int writePoint;
char msg1[]={"XXXXXXXXX\0"};
unsigned char dotpat[8];
int	lcdData[16];

// Timer subroutine
void __attribute__((__interrupt__, __shadow__, no_auto_psv)) _AltT3Interrupt(void)
{
	int i;
	int val;
	int topbar[16];
	int btmbar[16];
	// --- A/D is 12bits, filtering is 12bits
	while(!IFS0bits.ADIF);	// wait ADC
	resultData = ReadADC12(0);	// 12bit resolution (16kHz sampling)

	indata = resultData - 2048;			// tɂ
	inputData[ writePoint ] = indata << 4; // ̓obt@ɓ

	if ( writePoint == fftPoints-1 ) {

		// f̎Ƀf[^A0
		for(i = 0; i < fftPoints; i++) {
			fftData[i].real = inputData[i];
			fftData[i].imag = 0;
		}

		// FFT̎siDSP֐j
		FFTComplexIP( powerOf2, fftData, (fractcomplex *)twiddleFactors,
		(int) __builtin_psvpage(&twiddleFactors[0]));

		// rbgt\[giDSP֐j
		BitReverseComplex( powerOf2, fftData ); 

		// f̐ΒliDSP֐j
		SquareMagnitudeCplx( fftPoints/2, fftData, powerSpec ); 

		if(PORTBbits.RB0 == 1) { // vbVXCb`̒lŎgI
			VectorCopy(16, lcdData, powerSpec); // 0`3750Hz܂ŕ\iDSP֐j
		} else {
			VectorCopy(16, lcdData, powerSpec+16); // 4000`7750Hz܂ŕ\iDSP֐j
		}

		// _OtLN^̊
		for(i = 0; i < 16; i++) {
			val = lcdData[i] / 10;
			if(val > 16) val = 16;
			if(val == 0) { topbar[i] = 32; btmbar[i] =0+8; }
			else if(val == 1) { topbar[i] = 32; btmbar[i] = 1+8; }
			else if(val == 2) { topbar[i] = 32; btmbar[i] = 2+8; }
			else if(val == 3) { topbar[i] = 32; btmbar[i] = 3+8; }
			else if(val == 4) { topbar[i] = 32; btmbar[i] = 4+8; }
			else if(val == 5) { topbar[i] = 32; btmbar[i] = 5+8; }
			else if(val == 6) { topbar[i] = 32; btmbar[i] = 6+8; }
			else if(val == 7) { topbar[i] = 32; btmbar[i] = 7+8; }
			else if(val == 8) { topbar[i] = 0+8; btmbar[i] = 7+8; }
			else if(val == 9) { topbar[i] = 1+8; btmbar[i] = 7+8; }
			else if(val == 10) { topbar[i] = 2+8; btmbar[i] = 7+8; }
			else if(val == 11) { topbar[i] = 3+8; btmbar[i] = 7+8; }
			else if(val == 12) { topbar[i] = 4+8; btmbar[i] = 7+8; }
			else if(val == 13) { topbar[i] = 5+8; btmbar[i] = 7+8; }
			else if(val == 14) { topbar[i] = 6+8; btmbar[i] = 7+8; }
			else { topbar[i] = 7+8; btmbar[i] = 7+8; }
		}

		// LCDɕ\
		sprintf(msg1,"%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c",
		topbar[0], topbar[1], topbar[2], topbar[3], topbar[4], topbar[5], topbar[6], topbar[7],
		topbar[8], topbar[9], topbar[10], topbar[11], topbar[12], topbar[13], topbar[14], topbar[15]);
		SetDDRamAddr(0x00);
		PutsXLCD(msg1);
		sprintf(msg1,"%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c",
		btmbar[0], btmbar[1], btmbar[2], btmbar[3], btmbar[4], btmbar[5], btmbar[6], btmbar[7],
		btmbar[8], btmbar[9], btmbar[10], btmbar[11], btmbar[12], btmbar[13], btmbar[14], btmbar[15]);
		SetDDRamAddr(0x40);
		PutsXLCD(msg1);
	}

	writePoint++;
	if ( writePoint == fftPoints )  {
		writePoint = 0;
	}

	IFS0bits.ADIF = 0;	// clear A/D interuppt flag
	IFS0bits.T3IF = 0;	// clear T3 interuppt flag
}

// Main routine
int main(void)
{
	// port init
	TRISB=0x01FF;
	ADPCFG = 0x03f5; // for in-circuit programmer
	
	// LCD init
	OpenXLCD(FOUR_BIT & TWO_LINE & SEG1_50_SEG51_100 & COM1_COM16);
	// turn on
	WriteCmdXLCD(DON & CURSOR_OFF & BLINK_OFF);

	SetDDRamAddr(0x00);

	// _OtLN^̍쐬(8)
	dotpat[0] = 0x00;
	dotpat[1] = 0x00;
	dotpat[2] = 0x00;
	dotpat[3] = 0x00;
	dotpat[4] = 0x00;
	dotpat[5] = 0x00;
	dotpat[6] = 0x00;
	dotpat[7] = 0x1F;
	lcd_putram(0, dotpat);//code=0, CGRAM data set

	dotpat[0] = 0x00;
	dotpat[1] = 0x00;
	dotpat[2] = 0x00;
	dotpat[3] = 0x00;
	dotpat[4] = 0x00;
	dotpat[5] = 0x00;
	dotpat[6] = 0x1F;
	dotpat[7] = 0x1F;
	lcd_putram(1, dotpat);//code=1, CGRAM data set

	dotpat[0] = 0x00;
	dotpat[1] = 0x00;
	dotpat[2] = 0x00;
	dotpat[3] = 0x00;
	dotpat[4] = 0x00;
	dotpat[5] = 0x1F;
	dotpat[6] = 0x1F;
	dotpat[7] = 0x1F;
	lcd_putram(2, dotpat);//code=2, CGRAM data set

	dotpat[0] = 0x00;
	dotpat[1] = 0x00;
	dotpat[2] = 0x00;
	dotpat[3] = 0x00;
	dotpat[4] = 0x1F;
	dotpat[5] = 0x1F;
	dotpat[6] = 0x1F;
	dotpat[7] = 0x1F;
	lcd_putram(3, dotpat);//code=3, CGRAM data set

	dotpat[0] = 0x00;
	dotpat[1] = 0x00;
	dotpat[2] = 0x00;
	dotpat[3] = 0x1F;
	dotpat[4] = 0x1F;
	dotpat[5] = 0x1F;
	dotpat[6] = 0x1F;
	dotpat[7] = 0x1F;
	lcd_putram(4, dotpat);//code=4, CGRAM data set

	dotpat[0] = 0x00;
	dotpat[1] = 0x00;
	dotpat[2] = 0x1F;
	dotpat[3] = 0x1F;
	dotpat[4] = 0x1F;
	dotpat[5] = 0x1F;
	dotpat[6] = 0x1F;
	dotpat[7] = 0x1F;
	lcd_putram(5, dotpat);//code=5, CGRAM data set

	dotpat[0] = 0x00;
	dotpat[1] = 0x1F;
	dotpat[2] = 0x1F;
	dotpat[3] = 0x1F;
	dotpat[4] = 0x1F;
	dotpat[5] = 0x1F;
	dotpat[6] = 0x1F;
	dotpat[7] = 0x1F;
	lcd_putram(6, dotpat);//code=6, CGRAM data set

	dotpat[0] = 0x1F;
	dotpat[1] = 0x1F;
	dotpat[2] = 0x1F;
	dotpat[3] = 0x1F;
	dotpat[4] = 0x1F;
	dotpat[5] = 0x1F;
	dotpat[6] = 0x1F;
	dotpat[7] = 0x1F;
	lcd_putram(7, dotpat);//code=7, CGRAM data set

	// Use Alternate Interrupt Vector Table
	INTCON2bits.ALTIVT=1;
	// Confirm to turn off ADC
	ADCON1bits.ADON=0;
	// ADC init
	_ADCHS=	ADC_CH0_POS_SAMPLEA_AN3 &
			ADC_CH0_NEG_SAMPLEA_NVREF;		// A-D channel-3 select
	SetChanADC12(_ADCHS);
	_ADCON1=ADC_MODULE_ON &
			ADC_IDLE_CONTINUE &
			ADC_FORMAT_INTG &
			ADC_CLK_TMR &
			ADC_AUTO_SAMPLING_ON &
			ADC_SAMP_OFF;
	_ADCON2=ADC_VREF_AVDD_AVSS &
			ADC_SCAN_OFF &
			ADC_SAMPLES_PER_INT_1 &
			ADC_ALT_BUF_OFF &
			ADC_ALT_INPUT_OFF;
	// Tad={Tcy(ADCS+1)}/2>334ns, Then ADCS>18.7, Tad=10*Tcy
	_ADCON3=ADC_SAMPLE_TIME_1 &
			ADC_CONV_CLK_SYSTEM &
			ADC_CONV_CLK_10Tcy;
	_ADPCFG=ENABLE_AN3_ANA;	// A-D channel-3 analog input
	_ADCSSL=SCAN_NONE;
	OpenADC12(_ADCON1, _ADCON2, _ADCON3, _ADPCFG, _ADCSSL);

	// Timer3 setting
	OpenTimer3(T3_ON & T3_GATE_OFF & T3_PS_1_1 & T3_SOURCE_INT, 1843-1);	// 16kHz sampling ADC
	ConfigIntTimer3(T3_INT_PRIOR_5 & T3_INT_ON);

	// enable interrupt
	DISICNT = 0x0000;

	writePoint = 0;

	// main loop
	while(1)
	{
		// A/D result store in resultData
	}
}

